import logging
import os
import posixpath
import re

from gettext import gettext as _
from subprocess import getstatusoutput

from checkbox.plugin import Plugin
from checkbox.properties import String
from checkbox.user_interface import PREV


class CertifyPrompt(Plugin):

    SUBMISSION_TEMPLATE = """\
The following report has been generated for submission to the Certification \
database:

  [[%s|View Report]]   [[%s|Spreadsheet Report]]

Please provide the Secure ID for this system:"""

    # Default secure ID which will be selected automatically.
    secure_id = String(default="")

    def register(self, manager):
        super(CertifyPrompt, self).register(manager)

        self.persist = None

        for (rt, rh) in [
             ("begin-persist", self.begin_persist),
             ("launchpad-report", self.launchpad_report),
             ("prompt-exchange", self.prompt_exchange),
             ("xls-report", self.xls_report),
             ("report-exchange", self.report_exchange)]:
            self._manager.reactor.call_on(rt, rh)

    def begin_persist(self, persist):
        self.persist = persist.root_at("exchange_prompt")

    def launchpad_report(self, report):
        self._launchpad_report = report

    def prompt_exchange(self, interface):
        """
        Display message while spreadsheet report is generated
        """
        if interface.direction != PREV:
            interface.show_progress(_("Generating spreadsheet report ..."),
                                    self._manager.reactor.fire, "xls-report")

        self._manager.reactor.fire("report-exchange", interface)

    def xls_report(self):
        (status, output) = getstatusoutput("xls_export")
        if status:
            logging.error("XLS Report:\n%s" % output)
        else:
            logging.info("XLS Report generated successfully %s" % output)

    def report_exchange(self, interface):
        if self.persist and self.persist.has("secure-id"):
            secure_id = self.persist.get("secure-id")
        else:
            secure_id = self.secure_id
        submitToHexr = False

        # Register temporary handler for exchange-error events
        errors = []

        def exchange_error(e):
            errors.append(e)

        event_id = self._manager.reactor.call_on("exchange-error",
                                                 exchange_error)

        while True:
            for error in errors:
                self._manager.reactor.fire("prompt-error",
                                           interface,
                                           error)

            url = "file://%s" % posixpath.abspath(self._launchpad_report)
            spreadsheet = "file://%s" \
                % "%(CHECKBOX_DATA)s/submission.xls" % os.environ
            (secure_id, submitToHexr) = interface.show_entry(_(self.SUBMISSION_TEMPLATE) \
                % (url, spreadsheet), secure_id, label='Secure ID:', showSubmitToHexr=True)

            if interface.direction == PREV:
                break

            if not re.match(r"^\S{15}$|^\S{18}$", secure_id, re.I):
                errors.append("Secure ID must be either 15 or 18 alphanumeric characters.")
                continue

            errors = []
            self._manager.reactor.fire("report-hardware-id", secure_id)
            self._manager.reactor.fire("report-submit-to-hexr", submitToHexr)
            interface.show_progress("Exchanging information with "
                                    "the server...",
                self._manager.reactor.fire, "hexr-exchange", interface)

            if not errors:
                break

        self._manager.reactor.cancel_call(event_id)
        if self.persist:
            self.persist.set("secure-id", secure_id)


factory = CertifyPrompt
