import json
import os
from gettext import gettext as _
from checkbox.plugin import Plugin
from checkbox.user_interface import PREV
from checkbox.properties import Int
from checkbox.job import PASS, UNINITIATED


class CertifyReport(Plugin):

    max_messages = Int(default=1000)

    package_keys = ("name", "version")
    device_keys = ("path", "bus", "category", "driver", "vendor_id",
        "product_id", "subvendor_id", "subproduct_id", "vendor", "product")
    dmi_keys = ("path", "category", "product", "vendor")

    def register(self, manager):
        super(CertifyReport, self).register(manager)

        for (rt, rh) in [
             ("prompt-report", self.prompt_report),
             ("report-json", self.report_json),
             ("report-attachments", self.report_attachments),
             ("report-cdimage", self.report_cdimage),
             ("report-cpuinfo", self.report_cpuinfo),
             ("report-device", self.report_device),
             ("report-dmi", self.report_dmi),
             ("report-efi", self.report_efi),
             ("report-lsb", self.report_lsb),
             ("report-dpkg", self.report_dpkg),
             ("report-meminfo", self.report_meminfo),
             ("report-package", self.report_package),
             ("report-tags", self.report_tags),
             ("report-suites", self.report_suites),
             ("report-tests", self.report_tests)]:
            self._manager.reactor.call_on(rt, rh)

        self._manager.reactor.call_on("report-attachments", self.pre_report_attachments, -100)

    def _convert_table(self, table, keys):
        return dict([(k,v) for k,v in table.items() if k in keys])

    def _exchange_devices(self, devices):
        if hasattr(self, "all_devices"):
            self.all_devices.extend(devices)
        else:
            self.all_devices = devices
        self._exchange_message("set-devices", {"devices": self.all_devices})

    def _exchange_message(self, type, message):
        message["type"] = type
        self._manager.reactor.fire("message-send", message)

    def prompt_report(self, interface):
        """
        Display message while report is generated
        """
        if interface.direction != PREV:
            interface.show_progress(_("Retrieving results ..."),
                                    self._manager.reactor.fire, "report")

        self._manager.reactor.fire("report-json", interface)

    def report_json(self, interface):
        """
        Export test results to a json file
        """
        report = {}

        def add_job(job):
            is_suite = 'type' in job and job['type'] == 'suite'
            if 'suite' in job:
                suite_name = job['suite']
                parent_node = add_job(self.suites[suite_name])

                if is_suite:
                    if job['description'] in parent_node:
                        return parent_node[job['description']]

                    node = {}
                    parent_node[job['description']] = node
                    return node
                parent_node[job['name']] = job
            else:
                if is_suite:
                    field = 'description'
                else:
                    field = 'name'

                if job[field] in report:
                    return report[job[field]]

                node = {}
                report[job[field]] = node
                return node

        for test in self.tests:
            add_job(test)

        f = open("%(CHECKBOX_DATA)s/JSON_report" % os.environ, 'w')
        with f:
            f.write(json.dumps(report, sort_keys=True, indent=4))
        try:
            interface.show_report("Test case results report", report)
        except NotImplementedError:
            # Silently ignore the interfaces that don't implement the method
            pass

    def pre_report_attachments(self, attachments):
        # Replace command with name to be consistent with direct submissions
        for attachment in attachments:
            attachment["command"] = attachment["name"]

    def report_attachments(self, attachments):
        messages = []
        for attachment in attachments:
            if attachment.get("status", UNINITIATED) == PASS:
                message = {
                    "name": attachment["name"],
                    "content": attachment["data"]}
                if "project" in attachment:
                    message["project"] = attachment["suite"]
                if "test" in attachment:
                    message["job"] = attachment["test"]
                messages.append(message)

        if messages:
            self._exchange_message("add-attachments", {"attachments": messages})

    def report_cdimage(self, resources):
        cdimage = resources[0]
        disk = dict(cdimage)
        self._exchange_message("set-disk", {"disk": disk})

    def report_cpuinfo(self, resources):
        keys = ("platform", "count", "type", "model", "model_number",
            "model_version", "model_revision", "cache", "bogomips",
            "speed", "other")
        cpuinfo = resources[0]
        processor = dict([(k,v) for k,v in cpuinfo.items() if k in keys])
        self._exchange_message("set-processor", {"processor": processor})

    def report_device(self, resources):
        devices = [self._convert_table(r, self.device_keys) for r in resources]
        self._exchange_devices(devices)

    def report_dmi(self, resources):
        devices = [self._convert_table(r, self.dmi_keys) for r in resources]
        for device in devices:
            device["bus"] = "dmi"
        self._exchange_devices(devices)

    def report_efi(self, resources):
        self.report_dmi(resources)

    def report_lsb(self, resources):
        lsb = resources[0]
        distribution = dict(lsb)
        self._exchange_message("set-distribution",
            {"distribution": distribution})

    def report_dpkg(self, resources):
        dpkg = resources[0]
        self._exchange_message("set-architecture",
            {"architecture": dpkg["architecture"]})

    def report_meminfo(self, resources):
        meminfo = resources[0]
        keys = ("total", "swap")
        memory = dict([(k,v) for k,v in meminfo.items() if k in keys])
        self._exchange_message("set-memory", {"memory": memory})

    def report_package(self, resources):
        packages = [self._convert_table(r, self.package_keys) for r in resources]
        self._exchange_message("set-packages", {"packages": packages})

    def report_tags(self, resources):
        tags = resources[0]
        self._exchange_message("set-tags", tags)

    def report_tests(self, tests):
        self.tests = tests
        messages = []
        for test in tests:
            message = {
                "type": test["type"],
                "name": test["name"],
                "project": test.get("suite"),
                "description": test["description"],
                "annotation": test.get("suite"),
                "depends": " ".join(test.get("depends", [])),
                "requires": " ".join(test.get("requires", [])),
                "timeout": test.get("timeout"),
                "command": test.get("command"),
                "environ": test.get("environ", []),
                "user": test.get("user")}
            messages.append(message)

        while messages:
            jobs = messages[:self.max_messages]
            messages = messages[self.max_messages:]
            self._exchange_message("add-jobs", {"jobs": jobs})

        messages = []
        for test in tests:
            devices = []
            packages = []
            # Note that using subsets to identify resource types is not
            # a particularly reliable solution.
            for resource in test.get("resources", []):
                if set(["bus", "path"]).issubset(list(resource.keys())):
                    devices.append(resource)
                elif set(["name", "version"]).issubset(list(resource.keys())):
                    packages.append(resource)

            if "suite" not in test:
                raise Exception("Test not part of test suite: %s" % test["name"])

            message = {
                "type": test["type"],
                "name": test["name"],
                "project": test["suite"],
                "status": test["status"],
                "value": test.get("data", ""),
                "devices": [self._convert_table(d, self.device_keys) for d in devices],
                "packages": [self._convert_table(p, self.package_keys) for p in packages]}
            messages.append(message)

        while messages:
            results = messages[:self.max_messages]
            messages = messages[self.max_messages:]
            self._exchange_message("add-results", {"results": results})

    def report_suites(self, suites):
        """
        Get tests results and store it
        to display them later
        """
        self.suites = dict([(suite['name'], suite) for suite in suites])


factory = CertifyReport
